@@ -1,47 +1,59 @@
-import functools
-import json
+import asyncio
 import os
 import sys
 from configparser import ConfigParser
-from enum import Enum
+from functools import wraps
+from pathlib import Path
+from typing import List
 
-import asyncclick as click
-from asyncclick import Context, UsageError
+import click
+from click import Context, UsageError
 from tortoise import Tortoise, generate_schema_for_client
 from tortoise.exceptions import OperationalError
 from tortoise.transactions import in_transaction
 from tortoise.utils import get_schema_sql
 
+from aerich.inspectdb import InspectDb
 from aerich.migrate import Migrate
-from aerich.utils import get_app_connection, get_app_connection_name, get_tortoise_config
+from aerich.utils import (
+    get_app_connection,
+    get_app_connection_name,
+    get_models_describe,
+    get_tortoise_config,
+    get_version_content_from_file,
+    write_version_file,
+)
 
 from . import __version__
+from .enums import Color
 from .models import Aerich
 
-
-class Color(str, Enum):
-    green = "green"
-    red = "red"
-    yellow = "yellow"
-
-
 parser = ConfigParser()
 
 
-def close_db(func):
-    @functools.wraps(func)
-    async def close_db_inner(*args, **kwargs):
-        result = await func(*args, **kwargs)
-        await Tortoise.close_connections()
-        return result
+def coro(f):
+    @wraps(f)
+    def wrapper(*args, **kwargs):
+        loop = asyncio.get_event_loop()
 
-    return close_db_inner
+        # Close db connections at the end of all all but the cli group function
+        try:
+            loop.run_until_complete(f(*args, **kwargs))
+        finally:
+            if f.__name__ != "cli":
+                loop.run_until_complete(Tortoise.close_connections())
+
+    return wrapper
 
 
 @click.group(context_settings={"help_option_names": ["-h", "--help"]})
 @click.version_option(__version__, "-V", "--version")
 @click.option(
-    "-c", "--config", default="aerich.ini", show_default=True, help="Config file.",
+    "-c",
+    "--config",
+    default="aerich.ini",
+    show_default=True,
+    help="Config file.",
 )
 @click.option("--app", required=False, help="Tortoise-ORM app name.")
 @click.option(
@@ -52,6 +64,7 @@ def close_db(func):
     help="Name of section in .ini file to use for aerich config.",
 )
 @click.pass_context
+@coro
 async def cli(ctx: Context, config, app, name):
     ctx.ensure_object(dict)
     ctx.obj["config_file"] = config
@@ -59,7 +72,7 @@ async def cli(ctx: Context, config, app, name):
 
     invoked_subcommand = ctx.invoked_subcommand
     if invoked_subcommand != "init":
-        if not os.path.exists(config):
+        if not Path(config).exists():
             raise UsageError("You must exec init first", ctx=ctx)
         parser.read(config)
 
@@ -68,85 +81,112 @@ async def cli(ctx: Context, config, app, name):
 
         tortoise_config = get_tortoise_config(ctx, tortoise_orm)
         app = app or list(tortoise_config.get("apps").keys())[0]
-        if "aerich.models" not in tortoise_config.get("apps").get(app).get("models"):
-            raise UsageError("Check your tortoise config and add aerich.models to it.", ctx=ctx)
         ctx.obj["config"] = tortoise_config
         ctx.obj["location"] = location
         ctx.obj["app"] = app
-
+        Migrate.app = app
         if invoked_subcommand != "init-db":
-            await Migrate.init_with_old_models(tortoise_config, app, location)
+            if not Path(location, app).exists():
+                raise UsageError("You must exec init-db first", ctx=ctx)
+            await Migrate.init(tortoise_config, app, location)
 
 
 @cli.command(help="Generate migrate changes file.")
 @click.option("--name", default="update", show_default=True, help="Migrate name.")
 @click.pass_context
-@close_db
+@coro
 async def migrate(ctx: Context, name):
-    config = ctx.obj["config"]
-    location = ctx.obj["location"]
-    app = ctx.obj["app"]
-
     ret = await Migrate.migrate(name)
     if not ret:
         return click.secho("No changes detected", fg=Color.yellow)
-    Migrate.write_old_models(config, app, location)
     click.secho(f"Success migrate {ret}", fg=Color.green)
 
 
-@cli.command(help="Upgrade to latest version.")
+@cli.command(help="Upgrade to specified version.")
 @click.pass_context
-@close_db
+@coro
 async def upgrade(ctx: Context):
     config = ctx.obj["config"]
     app = ctx.obj["app"]
     migrated = False
-    for version in Migrate.get_all_version_files():
+    for version_file in Migrate.get_all_version_files():
         try:
-            exists = await Aerich.exists(version=version, app=app)
+            exists = await Aerich.exists(version=version_file, app=app)
         except OperationalError:
             exists = False
         if not exists:
             async with in_transaction(get_app_connection_name(config, app)) as conn:
-                file_path = os.path.join(Migrate.migrate_location, version)
-                with open(file_path, "r", encoding="utf-8") as f:
-                    content = json.load(f)
-                    upgrade_query_list = content.get("upgrade")
-                    for upgrade_query in upgrade_query_list:
-                        await conn.execute_script(upgrade_query)
-                await Aerich.create(version=version, app=app)
-            click.secho(f"Success upgrade {version}", fg=Color.green)
+                file_path = Path(Migrate.migrate_location, version_file)
+                content = get_version_content_from_file(file_path)
+                upgrade_query_list = content.get("upgrade")
+                for upgrade_query in upgrade_query_list:
+                    await conn.execute_script(upgrade_query)
+                await Aerich.create(
+                    version=version_file,
+                    app=app,
+                    content=get_models_describe(app),
+                )
+            click.secho(f"Success upgrade {version_file}", fg=Color.green)
             migrated = True
     if not migrated:
-        click.secho("No migrate items", fg=Color.yellow)
+        click.secho("No upgrade items found", fg=Color.yellow)
 
 
-@cli.command(help="Downgrade to previous version.")
+@cli.command(help="Downgrade to specified version.")
+@click.option(
+    "-v",
+    "--version",
+    default=-1,
+    type=int,
+    show_default=True,
+    help="Specified version, default to last.",
+)
+@click.option(
+    "-d",
+    "--delete",
+    is_flag=True,
+    default=False,
+    show_default=True,
+    help="Delete version files at the same time.",
+)
 @click.pass_context
-@close_db
-async def downgrade(ctx: Context):
+@click.confirmation_option(
+    prompt="Downgrade is dangerous, which maybe lose your data, are you sure?",
+)
+@coro
+async def downgrade(ctx: Context, version: int, delete: bool):
     app = ctx.obj["app"]
     config = ctx.obj["config"]
-    last_version = await Migrate.get_last_version()
-    if not last_version:
-        return click.secho("No last version found", fg=Color.yellow)
-    file = last_version.version
-    async with in_transaction(get_app_connection_name(config, app)) as conn:
-        file_path = os.path.join(Migrate.migrate_location, file)
-        with open(file_path, "r", encoding="utf-8") as f:
-            content = json.load(f)
+    if version == -1:
+        specified_version = await Migrate.get_last_version()
+    else:
+        specified_version = await Aerich.filter(app=app, version__startswith=f"{version}_").first()
+    if not specified_version:
+        return click.secho("No specified version found", fg=Color.yellow)
+    if version == -1:
+        versions = [specified_version]
+    else:
+        versions = await Aerich.filter(app=app, pk__gte=specified_version.pk)
+    for version in versions:
+        file = version.version
+        async with in_transaction(get_app_connection_name(config, app)) as conn:
+            file_path = Path(Migrate.migrate_location, file)
+            content = get_version_content_from_file(file_path)
             downgrade_query_list = content.get("downgrade")
             if not downgrade_query_list:
-                return click.secho("No downgrade item found", fg=Color.yellow)
+                click.secho("No downgrade items found", fg=Color.yellow)
+                return
             for downgrade_query in downgrade_query_list:
                 await conn.execute_query(downgrade_query)
-            await last_version.delete()
-        return click.secho(f"Success downgrade {file}", fg=Color.green)
+            await version.delete()
+            if delete:
+                os.unlink(file_path)
+            click.secho(f"Success downgrade {file}", fg=Color.green)
 
 
 @cli.command(help="Show current available heads in migrate location.")
 @click.pass_context
-@close_db
+@coro
 async def heads(ctx: Context):
     app = ctx.obj["app"]
     versions = Migrate.get_all_version_files()
@@ -156,12 +196,12 @@ async def heads(ctx: Context):
             click.secho(version, fg=Color.green)
             is_heads = True
     if not is_heads:
-        click.secho("No available heads,try migrate", fg=Color.green)
+        click.secho("No available heads,try migrate first", fg=Color.green)
 
 
 @cli.command(help="List all migrate items.")
 @click.pass_context
-@close_db
+@coro
 async def history(ctx: Context):
     versions = Migrate.get_all_version_files()
     for version in versions:
@@ -178,15 +218,21 @@ async def history(ctx: Context):
     help="Tortoise-ORM config module dict variable, like settings.TORTOISE_ORM.",
 )
 @click.option(
-    "--location", default="./migrations", show_default=True, help="Migrate store location."
+    "--location",
+    default="./migrations",
+    show_default=True,
+    help="Migrate store location.",
 )
 @click.pass_context
+@coro
 async def init(
-    ctx: Context, tortoise_orm, location,
+    ctx: Context,
+    tortoise_orm,
+    location,
 ):
     config_file = ctx.obj["config_file"]
     name = ctx.obj["name"]
-    if os.path.exists(config_file):
+    if Path(config_file).exists():
         return click.secho("You have inited", fg=Color.yellow)
 
     parser.add_section(name)
@@ -196,8 +242,7 @@ async def init(
     with open(config_file, "w", encoding="utf-8") as f:
         parser.write(f)
 
-    if not os.path.isdir(location):
-        os.mkdir(location)
+    Path(location).mkdir(parents=True, exist_ok=True)
 
     click.secho(f"Success create migrate location {location}", fg=Color.green)
     click.secho(f"Success generate config file {config_file}", fg=Color.green)
@@ -212,20 +257,20 @@ async def init(
     show_default=True,
 )
 @click.pass_context
-@close_db
+@coro
 async def init_db(ctx: Context, safe):
     config = ctx.obj["config"]
     location = ctx.obj["location"]
     app = ctx.obj["app"]
 
-    dirname = os.path.join(location, app)
-    if not os.path.isdir(dirname):
-        os.mkdir(dirname)
+    dirname = Path(location, app)
+    try:
+        dirname.mkdir(parents=True)
         click.secho(f"Success create app migrate location {dirname}", fg=Color.green)
-    else:
-        return click.secho(f"Inited {app} already", fg=Color.yellow)
-
-    Migrate.write_old_models(config, app, location)
+    except FileExistsError:
+        return click.secho(
+            f"Inited {app} already, or delete {dirname} and try again.", fg=Color.yellow
+        )
 
     await Tortoise.init(config=config)
     connection = get_app_connection(config, app)
@@ -234,15 +279,41 @@ async def init_db(ctx: Context, safe):
     schema = get_schema_sql(connection, safe)
 
     version = await Migrate.generate_version()
-    await Aerich.create(version=version, app=app)
-    with open(os.path.join(dirname, version), "w", encoding="utf-8") as f:
-        content = {
-            "upgrade": [schema],
-        }
-        json.dump(content, f, ensure_ascii=False, indent=2)
-    return click.secho(f'Success generate schema for app "{app}"', fg=Color.green)
+    await Aerich.create(
+        version=version,
+        app=app,
+        content=get_models_describe(app),
+    )
+    content = {
+        "upgrade": [schema],
+    }
+    write_version_file(Path(dirname, version), content)
+    click.secho(f'Success generate schema for app "{app}"', fg=Color.green)
+
+
+@cli.command(help="Introspects the database tables to standard output as TortoiseORM model.")
+@click.option(
+    "-t",
+    "--table",
+    help="Which tables to inspect.",
+    multiple=True,
+    required=False,
+)
+@click.pass_context
+@coro
+async def inspectdb(ctx: Context, table: List[str]):
+    config = ctx.obj["config"]
+    app = ctx.obj["app"]
+    connection = get_app_connection(config, app)
+
+    inspect = InspectDb(connection, table)
+    await inspect.inspect()
 
 
 def main():
     sys.path.insert(0, ".")
-    cli(_anyio_backend="asyncio")
+    cli()
+
+
+if __name__ == "__main__":
+    main()
